/**
Demo of critical bands with noise
David Gordon

Key commands:
'o': turns sine tone on/off
'r': return to original frequency
*/

import ddf.minim.*;
import ddf.minim.ugens.*;
import ddf.minim.effects.*;

Minim       minim;
AudioOutput out;
BandPass  bpFilter1, bpFilter2;
Oscil       wave1;

float startFreq = 2000.0;
float osc2Freq = startFreq;
float maxDist = 180;
float startBandwidth = 10;
float maxBandwidthDist = 500;
float noiseBandwidth = startBandwidth;
boolean waveOff = false;

Noise noise1, noise2;

void setup()
{
  size(512, 200, P3D);
  
  minim = new Minim(this);

  noise1 = new Noise( 0.75f, Noise.Tint.WHITE );
  wave1  = new Oscil( startFreq, 0.1f, Waves.SINE );

  // create filters for the noise
  bpFilter1 = new BandPass(430, noiseBandwidth, 44100);
  bpFilter2 = new BandPass(430, noiseBandwidth, 44100);

  // use the getLineOut method of the Minim object to get AudioOutput object
  out = minim.getLineOut();
  
  // patch the Oscil to the output
  noise1.patch( bpFilter1 ).patch(out);
//  noise2.patch( bpFilter2 ).patch(out);
  wave1.patch( out );
}

void draw()
{
  background(0);
  
  stroke(255, 255, 255);
  strokeWeight(1);
  
  // draw waveform
  for(int i = 0; i < out.bufferSize() - 1; i++)
  {
    line( i, 50  - out.left.get(i)*100,  i+1, 50  - out.left.get(i+1)*100 );
    line( i, 150 - out.right.get(i)*100, i+1, 150 - out.right.get(i+1)*100 );
  }
   text("Frequency 1:"+startFreq, 50, 190);
   text("Frequency 2:"+osc2Freq, 200, 190);
   text("Noise bandwidth:"+noiseBandwidth, 350, 190);

}


void keyPressed()
{ 
  switch( key )
  {
    case 'o':
      waveOff = !waveOff;
      if(waveOff)
        wave1.setAmplitude(0);
      else
        wave1.setAmplitude(0.075);
      break;
     
    case 'r':
     osc2Freq = startFreq;
     wave1.setFrequency(osc2Freq);
      break;
     
    default:
      break; 
  }
}

void mousePressed()
{
   osc2Freq = constrain( map( height-mouseY, 0, height, startFreq, startFreq+maxDist ), startFreq, startFreq+maxDist );
   wave1.setFrequency(osc2Freq);
   noiseBandwidth = constrain( map( mouseX, 0, width, startBandwidth, startBandwidth+maxBandwidthDist ), startBandwidth, startBandwidth+maxBandwidthDist );
   bpFilter1.setBandWidth(noiseBandwidth);
}

void mouseDragged()
{
   osc2Freq = constrain( map( height-mouseY, 0, height, startFreq, startFreq+maxDist ), startFreq, startFreq+maxDist );
   wave1.setFrequency(osc2Freq);
   noiseBandwidth = constrain( map( mouseX, 0, width, startBandwidth, startBandwidth+maxBandwidthDist ), startBandwidth, startBandwidth+maxBandwidthDist );
   bpFilter1.setBandWidth(noiseBandwidth);
}

